# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

from typing import Optional

from cleez.colors import bold
from cleez.command import Argument, Command

from ..shell import run
from ._util import check_files_exist


class TestCommand(Command):
    """Run tests."""

    name = "test"

    arguments = [
        Argument("args", nargs="?", help="Files or directories to test"),
    ]

    def run(self, args: Optional[list[str]] = None):
        print(bold("Running tests..."))
        if not args:
            args = ["src", "tests"]

        check_files_exist(args)

        args_str = " ".join(args)
        run(f"pytest {args_str}")



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

from typing import Optional

from cleez.colors import bold
from cleez.command import Argument, Command

from ..shell import run
from ._util import check_files_exist


class FormatCommand(Command):
    """Format code in specified files or directories."""

    name = "format"

    arguments = [
        Argument("args", nargs="*", help="Files or directories to format"),
    ]

    def run(self, args: Optional[list[str]] = None):
        print(bold("Formatting code..."))
        if not args:
            args = ["src", "tests"]

        check_files_exist(args)

        args_str = " ".join(args)

        run(f"black {args_str}")
        run(f"isort {args_str}")



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

from cleez.command import Command

from .check import CheckCommand
from .test import TestCommand

CRUFT_DIRS = [".mypy_cache", ".pytest_cache", ".ruff_cache"]


class AllCommand(Command):
    """Run everything (linters and tests)."""

    name = "all"

    def run(self):
        CheckCommand(self.cli).run()
        TestCommand(self.cli).run()



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT
from pathlib import Path
from typing import Optional

from cleez.colors import bold
from cleez.command import Argument, Command

from ..shell import run
from ._util import check_files_exist


class CheckCommand(Command):
    """Run checker/linters on specified files or directories."""

    name = "check"

    arguments = [
        Argument("args", nargs="*", help="Files or directories to check"),
    ]

    def run(self, args: Optional[list[str]] = None):
        print(bold("Running checks..."))
        if not args:
            args = ["src", "tests"]

        check_files_exist(args)

        args_str = " ".join(args)

        # Todo: more cases
        if Path("etc").joinpath("ruff.toml").exists():
            run(f"ruff check -c etc/ruff.toml {args_str}")
        else:
            run(f"ruff check {args_str}")

        run(f"flake8 {args_str}")
        run(f"mypy --show-error-codes {args_str}")
        run("pyright")
        run(f"vulture --min-confidence 80 {args_str}")
        # TODO: currently broken
        # run("deptry .")



"""Version bumpers, using date-based versioning.

Todo: also support semantic versionning.
"""

import sys

from invoke import Context

from abilian_devtools.commands.bump import get_version, update_version

TASKS = ["bump_version"]


def bump_version(c: Context, rule: str = "patch"):
    """Bump version in pyproject.toml, commit & apply tag.

    Parameter "rule" can be one of: "daily" or a parameter accepted by
    "poetry version".
    """
    r = c.run("git diff --quiet", echo=True, warn=True)
    if r.exited != 0:
        print("git diff --quiet failed")
        sys.exit()

    update_version(rule)
    version = get_version()
    c.run(f"git tag {version}", echo=True)
    c.run("git add pyproject.toml", echo=True)
    c.run(f"git commit -m 'Bump version ({version})'", echo=True)



# SPDX-FileCopyrightText: 2023 2022-2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT
import shlex
import subprocess
import sys

from cleez.colors import dim, red


def run(cmd, echo=True, warn=False):
    if echo:
        print(dim("> " + cmd))
    args = shlex.split(cmd)
    p = subprocess.Popen(args)
    p.wait()
    if not warn and p.returncode:
        print(red(f"failed with error code {p.returncode}"))
        sys.exit()
    return p.returncode



# SPDX-FileCopyrightText: 2023 Abilian SAS <https://abilian.com/>
#
# SPDX-License-Identifier: MIT

from __future__ import annotations

import importlib.metadata

from cleez import CLI
from cleez.actions import VERSION


def main():
    cli = get_cli()
    cli.run()


def get_cli():
    version = importlib.metadata.version("abilian-devtools")
    cli = CLI(name="adt", version=version)
    cli.add_option(
        "-V",
        "--version",
        action=VERSION,
        version=cli.version,
        help="Show version and exit",
    )
    cli.add_option(
        "-d", "--debug", default=False, action="store_true", help="Enable debug mode"
    )
    cli.add_option(
        "-v", "--verbose", default=False, action="store_true", help="Increase verbosity"
    )
    cli.scan("abilian_devtools.commands")
    return cli
